@@ -34,7 +34,7 @@ from django.template.response import SimpleTemplateResponse, TemplateResponse
 from django.urls import re_path
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_str
-from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext as _
 from django.views.decorators.http import require_POST
 
 from cms import operations
@@ -50,6 +50,7 @@ from cms.admin.forms import (
 )
 from cms.admin.permissionadmin import PERMISSION_ADMIN_INLINES
 from cms.cache.permissions import clear_permission_cache
+from cms.constants import MODAL_HTML_REDIRECT
 from cms.models import (
     CMSPlugin,
     EmptyPageContent,
@@ -65,6 +66,7 @@ from cms.operations.helpers import (
 )
 from cms.plugin_pool import plugin_pool
 from cms.signals.apphook import set_restart_trigger
+from cms.toolbar.utils import get_object_edit_url
 from cms.utils import get_current_site, page_permissions, permissions
 from cms.utils.admin import jsonify_request
 from cms.utils.conf import get_cms_setting
@@ -127,11 +129,7 @@ class PageAdmin(admin.ModelAdmin):
         return
 
     def get_admin_url(self, action, *args):
-        url_name = "{}_{}_{}".format(
-            self.opts.app_label,
-            self.opts.model_name,
-            action,
-        )
+        url_name = f"{self.opts.app_label}_{self.opts.model_name}_{action}"
         return admin_reverse(url_name, args=args)
 
     def get_preserved_filters(self, request):
@@ -281,7 +279,7 @@ class PageAdmin(admin.ModelAdmin):
             raise self._get_404_exception(object_id)
 
         if not page.is_potential_home():
-            return HttpResponseBadRequest(force_str(_("The page is not eligible to be home.")))
+            return HttpResponseBadRequest(_("The page is not eligible to be home."))
 
         new_home_tree, old_home_tree = page.set_as_homepage(request.user)
 
@@ -371,13 +369,6 @@ class PageAdmin(admin.ModelAdmin):
             **get_deleted_objects_additional_kwargs
         )
 
-        # This is bad and I should feel bad.
-        if _('placeholder') in perms_needed:
-            perms_needed.remove('placeholder')
-
-        if _('page content') in perms_needed:
-            perms_needed.remove('page content')
-
         if request.POST and not protected:  # The user has confirmed the deletion.
             if perms_needed:
                 raise PermissionDenied
@@ -459,7 +450,7 @@ class PageAdmin(admin.ModelAdmin):
 
         # Delete all of the pages titles contents
         ct_page_content = ContentType.objects.get_for_model(PageContent)
-        page_content_objs = PageContent.objects.filter(page__in=cms_pages)
+        page_content_objs = PageContent.admin_manager.filter(page__in=cms_pages).values_list('pk', flat=True)
         placeholders = Placeholder.objects.filter(
             content_type=ct_page_content,
             object_id__in=page_content_objs,
@@ -601,8 +592,8 @@ class PageAdmin(admin.ModelAdmin):
 
         # Does the user have permissions to do this...?
         if not can_move_page or (target and not target.has_add_permission(user)):
-            message = force_str(
-                _("Error! You don't have permissions to move this page. Please reload the page")
+            message = _(
+                "Error! You don't have permissions to move this page. Please reload the page"
             )
             return jsonify_request(HttpResponseForbidden(message))
 
@@ -654,13 +645,14 @@ class PageAdmin(admin.ModelAdmin):
         )
 
         if not can_change_global_permissions:
-            allowed_pages = frozenset(page_permissions.get_change_id_list(user, site, check_global=False))
+            allowed_pages = page_permissions.get_change_perm_tuples(user, site, check_global=False)
 
         for permission in _page_permissions.iterator():
             if can_change_global_permissions:
                 can_change = True
             else:
-                can_change = permission.page_id in allowed_pages
+                page_path = permission.page.node.path
+                can_change = any(perm_tuple.contains(page_path) for perm_tuple in allowed_pages)
 
             row = PermissionRow(
                 is_global=False,
@@ -710,15 +702,17 @@ class PageAdmin(admin.ModelAdmin):
             can_copy_page = page_permissions.user_can_add_page(user, site)
 
         if not can_copy_page:
-            message = force_str(_("Error! You don't have permissions to copy this page."))
+            message = _("Error! You don't have permissions to copy this page.")
             return jsonify_request(HttpResponseForbidden(message))
 
         page_languages = page.get_languages()
         site_languages = get_language_list(site_id=site.pk)
 
         if not any(lang in page_languages for lang in site_languages):
-            message = force_str(_("Error! The page you're pasting is not "
-                                  "translated in any of the languages configured by the target site."))
+            message = _(
+                "Error! "
+                "The page you're pasting is not translated in any of the languages configured by the target site."
+            )
             return jsonify_request(HttpResponseBadRequest(message))
 
         new_page = form.copy_page(user)
@@ -726,10 +720,10 @@ class PageAdmin(admin.ModelAdmin):
 
     def edit_title_fields(self, request, page_id, language):
         page = self.get_object(request, object_id=page_id)
-        translation = page.get_content_obj(language, fallback=False)
+        translation = page.get_admin_content(language)
 
         if not self.has_change_permission(request, obj=page):
-            return HttpResponseForbidden(force_str(_("You do not have permission to edit this page")))
+            return HttpResponseForbidden(_("You do not have permission to edit this page"))
 
         if page is None:
             raise self._get_404_exception(page_id)
@@ -828,15 +822,11 @@ class PageContentAdmin(admin.ModelAdmin):
         obj = super().get_object(request, object_id, from_field)
 
         if obj:
-            obj.page.page_content_cache[obj.language] = obj
+            obj.page.admin_content_cache[obj.language] = obj
         return obj
 
     def get_admin_url(self, action, *args):
-        url_name = "{}_{}_{}".format(
-            self.opts.app_label,
-            self.opts.model_name,
-            action,
-        )
+        url_name = f"{self.opts.app_label}_{self.opts.model_name}_{action}"
         return admin_reverse(url_name, args=args)
 
     def get_preserved_filters(self, request):
@@ -990,6 +980,20 @@ class PageContentAdmin(admin.ModelAdmin):
 
         return super().change_view(request, object_id, form_url=form_url, extra_context=context)
 
+    def response_add(self, request, obj):
+        redirect = request.POST.get("edit", False)
+        if redirect == "1":
+            from django.core.cache import cache
+
+            from cms.cache.permissions import get_cache_key, get_cache_permission_version
+            cache.delete(get_cache_key(request.user, 'change_page'), version=get_cache_permission_version())
+
+            # redirect to the edit view if added from the toolbar
+            url = get_object_edit_url(obj)  # Redirects to preview if necessary
+            return HttpResponse(MODAL_HTML_REDIRECT.format(url=url))
+        return super().response_add(request, obj)
+
+
     def get_filled_languages(self, request, page):
         site_id = get_site(request).pk
         filled_languages = page.get_languages()
@@ -1043,11 +1047,29 @@ class PageContentAdmin(admin.ModelAdmin):
             return page_permissions.user_can_change_page(request.user, page=obj.page, site=site)
         can_change_page = page_permissions.user_can_change_at_least_one_page(
             user=request.user,
-            site=get_site(request),
+            site=site,
             use_cache=False,
         )
         return can_change_page
 
+    def has_view_permission(self, request, obj=None):
+        """
+        Return true if the current user has permission on the page.
+        Return the string 'All' if the user has all rights.
+        """
+        # Identical to has_change_permission, but will remain untouched by any subclassing
+        # as done, e.g., by djangocms-versioning
+        site = get_site(request)
+
+        if obj:
+            return page_permissions.user_can_change_page(request.user, page=obj.page, site=site)
+        can_view_page = page_permissions.user_can_change_at_least_one_page(
+            user=request.user,
+            site=get_site(request),
+            use_cache=False,
+        )
+        return can_view_page
+
     def has_delete_permission(self, request, obj=None):
         """
         Returns True if the current user has permission to delete the page.
@@ -1123,7 +1145,7 @@ class PageContentAdmin(admin.ModelAdmin):
             Prefetch(
                 'pagecontent_set',
                 to_attr='filtered_translations',
-                queryset=self.get_queryset(request),
+                queryset=page_contents,
             ),
         )
 
@@ -1174,12 +1196,12 @@ class PageContentAdmin(admin.ModelAdmin):
         to_template = request.POST.get("template", None)
 
         if to_template not in dict(get_cms_setting('TEMPLATES')):
-            return HttpResponseBadRequest(force_str(_("Template not valid")))
+            return HttpResponseBadRequest(_("Template not valid"))
 
         page_content.template = to_template
         page_content.save()
 
-        return HttpResponse(force_str(_("The template was successfully changed")))
+        return HttpResponse(_("The template was successfully changed"))
 
     @require_POST
     @transaction.atomic
@@ -1196,7 +1218,7 @@ class PageContentAdmin(admin.ModelAdmin):
         page = source_page_content.page
 
         if not target_language or target_language not in get_language_list(site_id=page.node.site_id):
-            return HttpResponseBadRequest(force_str(_("Language must be set to a supported language!")))
+            return HttpResponseBadRequest(_("Language must be set to a supported language!"))
 
         target_page_content = page.get_content_obj(target_language, fallback=False)
 
@@ -1206,7 +1228,7 @@ class PageContentAdmin(admin.ModelAdmin):
             plugins = placeholder.get_plugins_list(source_page_content.language)
 
             if not target.has_add_plugins_permission(request.user, plugins):
-                return HttpResponseForbidden(force_str(_('You do not have permission to copy these plugins.')))
+                return HttpResponseForbidden(_("You do not have permission to copy these plugins."))
             copy_plugins_to_placeholder(plugins, target, language=target_language)
         return HttpResponse("ok")
 
@@ -1219,7 +1241,7 @@ class PageContentAdmin(admin.ModelAdmin):
         request_language = get_site_language_from_request(request, site_id=page.node.site_id)
 
         if not self.has_delete_translation_permission(request, language, page):
-            return HttpResponseForbidden(force_str(_("You do not have permission to delete this page")))
+            return HttpResponseForbidden(_("You do not have permission to delete this page"))
 
         if page is None:
             raise self._get_404_exception(object_id)
@@ -1255,13 +1277,6 @@ class PageContentAdmin(admin.ModelAdmin):
             list(perms_needed_url) + list(perms_needed_translation) + list(perms_needed_plugins)
         )
 
-        # This is bad and I should feel bad.
-        if 'placeholder' in perms_needed:
-            perms_needed.remove('placeholder')
-
-        if 'page content' in perms_needed:
-            perms_needed.remove('page content')
-
         if request.method == 'POST':
             if perms_needed:
                 raise PermissionDenied
@@ -1299,7 +1314,7 @@ class PageContentAdmin(admin.ModelAdmin):
                 return HttpResponseRedirect(admin_reverse('index'))
 
             redirect_to = self.get_admin_url('changelist')
-            redirect_to += '?language={}'.format(request_language)
+            redirect_to += f'?language={request_language}'
             return HttpResponseRedirect(redirect_to)
 
         context = {
@@ -1330,11 +1345,11 @@ class PageContentAdmin(admin.ModelAdmin):
         if not self.has_change_permission(request, obj=page_content):
             if self.has_change_permission(request):
                 # General (permission) problem
-                message = _("You do not have permission to change a page's navigation status")
+                message = "You do not have permission to change a page's navigation status"
             else:
                 # Only this page? Can be permissions or versioning, or ...
-                message = _("You cannot change this page's navigation status")
-            return HttpResponseForbidden(force_str(message))
+                message = "You cannot change this page's navigation status"
+            return HttpResponseForbidden(_(message))
 
         if page_content is None:
             raise self._get_404_exception(object_id)
@@ -1373,7 +1388,7 @@ class PageContentAdmin(admin.ModelAdmin):
             Prefetch(
                 'pagecontent_set',
                 to_attr='filtered_translations',
-                queryset=self.get_queryset(request),
+                queryset=PageContent.admin_manager.get_queryset(),
             ),
         )
         rows = self.get_tree_rows(
@@ -1383,7 +1398,7 @@ class PageContentAdmin(admin.ModelAdmin):
             depth=(page.node.depth + 1 if page else 1),
             follow_descendants=True,
         )
-        return HttpResponse(u''.join(rows))
+        return HttpResponse(''.join(rows))
 
     def get_tree_rows(self, request, pages, language, depth=1,
                       follow_descendants=True):
@@ -1402,13 +1417,7 @@ class PageContentAdmin(admin.ModelAdmin):
         user_can_change_advanced = page_permissions.user_can_change_page_advanced_settings
 
         def render_page_row(page):
-            page.page_content_cache = {trans.language: trans for trans in page.filtered_translations}
-
-            for _language in languages:
-                # EmptyPageContent is used to prevent the cms from trying
-                # to find a translation in the database
-                page.page_content_cache.setdefault(_language, EmptyPageContent(language=_language, page=page))
-
+            page.admin_content_cache = {trans.language: trans for trans in page.filtered_translations}
             has_move_page_permission = page_permissions.user_can_move_page(request.user, page, site=site)
 
             if permissions_on and not has_move_page_permission:
@@ -1422,7 +1431,7 @@ class PageContentAdmin(admin.ModelAdmin):
                 'opts': self.opts,
                 'site': site,
                 'page': page,
-                'page_content': page.get_content_obj(language, fallback=False),  # Show specific language
+                'page_content': page.get_admin_content(language),
                 'node': page.node,
                 'ancestors': [node.item for node in page.node.get_cached_ancestors()],
                 'descendants': [node.item for node in page.node.get_cached_descendants()],
